
-- =====   GLOBALS =======

Projects = {} --stores each table containing sub projects
Canvas = {}
Canvas.x = 0
Canvas.y = 0
Canvas.scale = 1


--[[
the global functions needed by the sub-projects
are stored seperatly so they can be written over in the
sub-projects environment
--]]

old = {} 
old.getPosition = love.mouse.getPosition
old.draw = love.graphics.draw
old.rect = love.graphics.rectangle
old.color = love.graphics.setColor
old.graphics = love.graphics
old.drawq = love.graphics.drawq
old.line = love.graphics.line
old.circle = love.graphics.circle
old.point = love.graphics.point
old.print = love.graphics.print
old.printf = love.graphics.printf
old.newImage = love.graphics.newImage
old.newFont = love.graphics.newFont
old.mouse = love.mouse
old.timer = love.timer
old.framebuffer = love.graphics.newFramebuffer or love.graphics.newCanvas
old.rendertarget = love.graphics.setRenderTarget

love.graphics.setBackgroundColor(40,40,100)

-- ====== LOCALS =======

local play = love.graphics.newImage("play.png")
local pause = love.graphics.newImage("pause.png")
local Settings = {}
Settings.AutoPause = true

function env_require(module, env) -- replacement require that loads the file in the correct environment
  local loaders = package.loaders
  local errors  = {}
  local load_fn
  for _, loader in ipairs(loaders) do
    local fn = loader(module)
    if type(fn) == 'function' then
      load_fn = fn
      break
    elseif type(fn) == 'string' then
      errors[#errors + 1] = fn
    end
  end

  if not load_fn then
    error(table.concat(errors, '\n'))
  end
  setfenv(load_fn, env)
  return load_fn(module)
end

local function ParseConf(proj, path)
    if not love.filesystem.exists(path.."conf.lua") then
        return 
    end

    local msg = love.filesystem.read(path.."conf.lua")

    proj.width = msg:match("screen.width -= -(%d+)") or proj.width
    proj.height = msg:match("screen.height -= -(%d+)") or proj.height
    proj.title = msg:match([=[title -= -['"](.-)['"]]=]) or proj.title

end



local function NewProject(path)
    local dir = "Projects/"..path.."/"

    local new = {} --new project
    new.env = {} --the environment of the project

    new.x = 0
    new.y = 0
    new.width = 800
    new.height = 600
    new.scalex = 0.2
    new.scaley = 0.2

    new.path = dir
    
    new.title = path

    ParseConf(new, dir) -- check the conf.lua for settings



    setmetatable(new.env, {__index = _G})
    setfenv(1, new.env)

    
    new.env.love = {}
    
    new.env.love.mouse = {}

    new.env.love.graphics = {}

    new.env.love.timer = old.timer

    setmetatable(new.env.love.graphics, new.env.love.graphics)
    new.env.love.graphics.__index = old.graphics   

     setmetatable(new.env.love.mouse, new.env.love.mouse)
    new.env.love.mouse.__index = old.mouse 
    
    function new.env.love.mouse.getPosition()
        local x,y = old.getPosition()
        
        x = (x - new.x*Canvas.scale-Canvas.x*Canvas.scale)/(new.scalex*Canvas.scale)
        y = (y - new.y*Canvas.scale-Canvas.y*Canvas.scale)/(new.scaley*Canvas.scale)

        x = math.max(x, 0)
        x = math.min(x, new.width)

        y = math.max(y, 0)
        y = math.min(y, new.height)
        return x,y
    end

    local function offx(x)
        x = x and (x*new.scalex*Canvas.scale)+new.x*Canvas.scale+Canvas.x*Canvas.scale
            or Canvas.x*Canvas.scale + new.x*Canvas.scale
        return x
    end

    local function offy(y)
        y = y and (y*new.scaley*Canvas.scale)+new.y*Canvas.scale+Canvas.y*Canvas.scale 
            or Canvas.y*Canvas.scale + new.y*Canvas.scale
        return y
    end

    local function scx(x)
        return x and x*new.scalex*Canvas.scale or new.scalex*Canvas.scale
    end

    local function scy(y)
        return y and y*new.scaley*Canvas.scale or new.scaley*Canvas.scale
    end


    --blank out a few functions so they cant be called
    --================================================
    
    function new.env.love.graphics.clear() end
    function new.env.love.graphics.push() end
    function new.env.love.graphics.pop() end
    function new.env.love.graphics.rotate() end
    function new.env.love.graphics.scale() end
    function new.env.love.graphics.toggleFullscreen() end
    function new.env.love.graphics.translate() end

    -- redefine other functions 
    --================================================

    function new.env.love.graphics.drawq(image, quad, x, y, r ,sx, sy, ox, oy)
        x = offx(x)
        y = offy(y)

        sx = scx(sx)
        sy = scy(sy)
        ox = ox and ox*new.scalex or ox --this is wrong
        oy = oy and oy*new.scaley or oy

        old.drawq(image,quad,x,y,r,sx,sy,ox,oy)
    end

    function new.env.love.graphics.getBackgroundColor()
        return unpack(new.bcolor or {0,0,0})
    end

    function new.env.love.graphics.getCaption()
        return new.caption
    end

    function new.env.love.graphics.getHeight()
        return new.height
    end

    function new.env.love.graphics.getWidth()
        return new.width
    end

    function new.env.love.graphics.line(...)
        local t = {...}
        for i = 1, #t,2 do
            t[i] = offx(t[i])
            t[i+1] = offy(t[i+1])
        end

        old.line(unpack(t))
    end

    function new.env.love.graphics.point(x,y)
        x = offx(x)
        y = offy(y)
        old.point(x,y)
    end

    function new.env.love.graphics.polygon(mode, points)
        for i = 1, #points, 2 do
            point[i] = offx(point[i])
            points[i+1] = offy(point[i+1])
        end
        old.polygon(mode, points)
    end


    function new.env.love.graphics.quad(mode,ulx,uly,urx,ury,blx,bly,brx,bry)
        ulx = offx(ulx)
        uly = offy(uly)
        urx = offx(ulx)
        ury = offy(uly)
        blx = offx(ulx)
        bly = offy(uly)
        brx = offx(ulx)
        bry = offy(uly)



        old.quad(mode, ulx,uly,urx,ury,blx,bly,brx,bry)
    end

    function new.env.love.graphics.setRenderTarget(targ)
        if targ then
            new.fb = true
        else
            new.fb = false
        end

        old.rendertarget(targ)
    end

    function new.env.love.graphics.print(text,x,y,r,sx,sy)
        x = offx(x)
        y = offy(y)

        sx = scx(sx)
        sy = scy(sy)
        old.print(text,x,y,r,sx,sy)
    end

    function new.env.love.graphics.printf(text, x, y, limit, align)
        x = offx(x)
        y = offy(f)

        limit = limit*new.scalex
        old.printf(text, x, y, limit,align)                
    end

    function new.env.love.graphics.draw(drawable, x,y,r,sx,sy,ox,oy)
        x = offx(x)
        y = offy(y)

        sx = scx(sx)
        sy = scy(sy)
        ox = ox and ox*new.scalex*Canvas.scale or ox
        oy = oy and oy*new.scaley*Canvas.scale or oy

        old.draw(drawable, x, y, r, sx, sy, ox, oy)
    end

    function new.env.love.graphics.rectangle(mode, x, y, width, height)
        if not new.fb then
            x = offx(x)
            y = offy(y)
            width = width*new.scalex*Canvas.scale
            height = height *new.scaley*Canvas.scale
        end
        old.rect(mode, x, y, width, height)
    end

    function new.env.love.graphics.newFramebuffer(...)
        return old.framebuffer(...)
    end

    function new.env.love.graphics.newCanvas(...)
        return old.framebuffer(...)
    end
    
    function new.env.love.graphics.triangle(mode, x1,y1,x2,y2,x3,y3)
        x1 = offx(x1)
        y1 = offy(y1)
        x2 = offx(x2)
        y2 = offy(y2)
        x3 = offx(x3)
        y3 = offy(y3)
        old.triangle(mode,x1,y1,x2,y2,x3,y3)
    end

    function new.env.love.graphics.circle(mode,x,y,radius,segments)
        x = offx(x)
        y = offy(y)
        radius = radius*((new.scalex+new.scaley)/2)
        old.circle(mode, x, y, radius, segments)
    end

    function new.env.love.graphics.setColor(...)
        old.color(...)
    end

    function new.env.require(path)
        env_require(dir..path, new.env)
    end

    function new.env.love.graphics.newImage(path)
        return old.newImage(new.path..path)
    end

    function new.env.love.graphics.newFont(path, size)
        return old.newFont(new.path..path, size)
    end

    function new.env.love.graphics.setBackgroundColor(r,g,b)
        --need to check they are all numbers
        new.bcolor = {r,g,b}
    end

    new.env.meep = "mpp"

    env_require(dir.."main", new.env)

    table.insert(Projects, new)
    return new
end

local proj = NewProject("test1")

proj.x = 50
proj.y = 50
proj.scaley = 0.2
proj.scalex = 0.2


proj = NewProject("test2")

proj.x = 300
proj.y = 300
proj.scalex = 0.3
proj.scaley = 0.3

proj = NewProject("fluid")
proj.x = 40
proj.y = 300

proj = NewProject("house test")
proj.x = 150
proj.y = 30
local x, y



local function canvx(x)
    return x*Canvas.scale+Canvas.x*Canvas.scale
end

local function canvy(y)
    return y*Canvas.scale+Canvas.y*Canvas.scale
end

function love.update(dt)
    for i = 1, #Projects do --handle updates to sub proj
        proj = Projects[i]
        if (not proj.paused) and not (Settings.AutoPause and not proj.selected)  then
            if proj.env.love.update then
                proj.env.love.update(dt)
            end
        end
        if proj.moving then
            x, y = love.mouse.getPosition()
            proj.x = x-proj.xoff
            proj.y = y-proj.yoff
        end
        if proj.sizing then
            x,y = love.mouse.getPosition()
            if proj.sizing == "right" then
                proj.scalex = (x-proj.x-Canvas.x)/proj.width
            elseif proj.sizing == "bottom" then
                proj.scaley = (y-proj.y-Canvas.y)/proj.height                
            end
        end
    end
    if Canvas.moving then
        x,y = love.mouse.getPosition()
        Canvas.x = x-Canvas.xoff
        Canvas.y = y-Canvas.yoff
    end
end



function love.mousepressed(x,y,button)
    
    local found

    for i = 1, #Projects do --handle updates to sub proj
         proj = Projects[i]
         local px,py = canvx(proj.x), canvy(proj.y)
        if x >= px-4*Canvas.scale 
            and x <= px + (proj.width*proj.scalex*Canvas.scale)+4*Canvas.scale 
            and y >= py-12*Canvas.scale 
            and y <= py +(proj.scaley*proj.height*Canvas.scale)+ 4*Canvas.scale then
            
            found = true
        proj.selected = true
        for j = 1, #Projects do
            if not (proj == Projects[j]) then
                Projects[j].selected = false
            end
        end
        if x >= proj.x 
            and x <= px + (proj.width*proj.scalex*Canvas.scale) 
            and y >= py 
            and y <= py +(proj.scaley*proj.height*Canvas.scale) then
            if proj.env.love.mousepressed then
                proj.env.love.mousepressed((x - px)/(proj.scalex*Canvas.scale), (y - py)/(proj.scaley*Canvas.scale),button)
            end
        end

            if x <= px then
                proj.sizing = "left"
            elseif x >= px+proj.width*proj.scalex*Canvas.scale then
                proj.sizing = "right"
            elseif y <= py then
                proj.moving = true
            elseif y >= py+proj.height*proj.scaley*Canvas.scale then
                proj.sizing = "bottom"
            end  
                
            proj.xoff = x-proj.x
            proj.yoff = y-proj.y
        end
    end

    if not found then
        if button == "l" then

        elseif button == "r" then

        elseif button == "m" then
            Canvas.xoff = x-Canvas.x
            Canvas.yoff = y-Canvas.y
            Canvas.moving = true
        elseif button == "wu" then
            Canvas.scale = Canvas.scale + 0.1
        elseif button == "wd" then
            Canvas.scale = Canvas.scale - 0.1
        end
    end
end


function love.mousereleased(x,y,button)

    for i = 1, #Projects do --handle updates to sub proj
         if x >= proj.x and x <= proj.x + (proj.width*proj.scalex) and y >= proj.y and y <=proj.y +(proj.scaley*proj.height) then
            if proj.env.love.mouserelease then
                proj.env.love.mousereleased((x - proj.x)/proj.scalex, (y - proj.y)/proj.scaley,button)
            end
        end

         proj = Projects[i]
         proj.moving = nil
         proj.sizing = nil
        if x >= proj.x and x <= proj.x+8 and y >= proj.y-8 and y <= proj.y then
            proj.paused = not proj.paused
        end
    end
    Canvas.moving = false
end

local proj



function love.draw()

    for i = 1, #Projects do -- handle draw to sub proj
        proj = Projects[i]
        if proj.env.love.draw then
            local x, y = canvx(proj.x), canvy(proj.y)
            love.graphics.setColor(0,0,0,255)
            love.graphics.rectangle("fill", x-4*Canvas.scale, y-12*Canvas.scale, 
                                    proj.width*proj.scalex*Canvas.scale+8*Canvas.scale,
                                    proj.height*proj.scaley*Canvas.scale+16*Canvas.scale)
            love.graphics.setColor(255,255,255,255)

            if proj.bcolor then
                love.graphics.setColor(proj.bcolor)
            else
                love.graphics.setColor(0,0,0,255)
            end
            
            love.graphics.rectangle("fill", x, y,
                                    proj.width*proj.scalex*Canvas.scale,
                                    proj.height*proj.scaley*Canvas.scale)
            
            love.graphics.setColor(255,255,255,255)

            proj.env.love.draw()
            love.graphics.setColor(255,255,255,255)
            if proj.paused or Settings.AutoPause and not proj.selected then
                love.graphics.draw(play, x,y-8*Canvas.scale, 0, 0.25*Canvas.scale, 0.25*Canvas.scale)
            else
                love.graphics.draw(pause, x,y-8*Canvas.scale, 0, 0.25*Canvas.scale, 0.25*Canvas.scale)
            end
            love.graphics.setColor(255,255,255,255)
            if proj.title then
                love.graphics.print(proj.title, x + 15*Canvas.scale,y-11*Canvas.scale, 0, 0.8*Canvas.scale, 0.8*Canvas.scale)
            end
        end
    end

	love.graphics.setColor(255,255,255,255)
	love.graphics.print(love.timer.getFPS(), 10, 10)   

end


